package swinggrapheditor.graph.xml;

import java.awt.EventQueue;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import swinggrapheditor.xmlutils.XmlMaker;
import java.io.Serializable;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;
import swinggrapheditor.db.DataSource;
import swinggrapheditor.graph.Arc;
import swinggrapheditor.graph.LocatedGraph;
import swinggrapheditor.graph.LocatedNode;

public class LocatedGraphXml<T extends Serializable> extends LocatedGraph<T> {

    public String getXml() throws ParserConfigurationException, TransformerConfigurationException, TransformerException {
        XmlMaker xmlMaker = new XmlMaker("locatedGraph");
        xmlMaker.insertElement("locatedNodes");
        for (LocatedNode<T> locatedNode : nodes) {
            xmlMaker.insertElement("locatedNode");
            xmlMaker.addAttribute("id", locatedNode.hashCode() + "");
            xmlMaker.addAttribute("x", locatedNode.x + "");
            xmlMaker.addAttribute("y", locatedNode.y + "");
            xmlMaker.insertText(locatedNode.element.toString());
            xmlMaker.stepOut();
        }
        xmlMaker.stepOut();
        xmlMaker.insertElement("arcs");
        for (Arc<T, LocatedNode<T>> arc : arcs) {
            xmlMaker.insertElement("arc");
            xmlMaker.addAttribute("edge1", arc.edge1.hashCode() + "");
            xmlMaker.addAttribute("edge2", arc.edge2.hashCode() + "");
            xmlMaker.stepOut();
        }
        xmlMaker.stepOut();
        return xmlMaker.getXmlAsString();
    }
    private final String nickname;

    public LocatedGraphXml(String nickname) {
        this.nickname = nickname;
    }

    public String saveToDataSource() throws ParserConfigurationException, SQLException, TransformerConfigurationException, TransformerException {
        String xml = getXml();
        DataSource.getInstance().saveGraph(nickname, xml);
        return xml;
    }

    public void loadFromDataSource() {
        new Thread() {
            @Override
            public void run() {
                String xmlString;
                try {
                    xmlString = DataSource.getInstance().loadGraph(nickname);
                    if (xmlString != null) {
                        parseXmlString(xmlString);
                    }
                } catch (ParserConfigurationException | SAXException | IOException | SQLException ex) {
                    System.err.println(ex.getMessage());
                }
                EventQueue.invokeLater(new Runnable() {

                    @Override
                    public void run() {
                        graphChanged();
                    }
                });
            }
        }.start();
    }

    private TextContentParser<T> tcp = null;

    public void setTextContentParser(TextContentParser<T> tcp) {
        this.tcp = tcp;
    }

    private void parseXmlString(String xmlString) throws ParserConfigurationException, SAXException, IOException {
        Document document = DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(new ByteArrayInputStream(xmlString.getBytes("UTF-8")));

        List<LocatedNode<T>> newNodes = new ArrayList<>();
        Map<Integer, LocatedNode<T>> newNodesMap = new TreeMap<>();
        NodeList locatedNodeNodeList = document.getElementsByTagName("locatedNode");
        for (int i = 0; i < locatedNodeNodeList.getLength(); i++) {
            Node locatedNodeNode = locatedNodeNodeList.item(i);
            if (locatedNodeNode.getNodeType() == Node.ELEMENT_NODE) {
                Element locatedNodeElement = (Element) locatedNodeNode;
                LocatedNode<T> newNode = new LocatedNode<>();
                newNode.x = Integer.parseInt(locatedNodeElement.getAttributeNode("x").getNodeValue());
                newNode.y = Integer.parseInt(locatedNodeElement.getAttributeNode("y").getNodeValue());
                if (tcp != null) {
                    newNode.element = tcp.parseTextContent(locatedNodeElement.getTextContent());
                }
                newNodes.add(newNode);
                newNodesMap.put(Integer.parseInt(locatedNodeElement.getAttribute("id")), newNode);
            }
        }

        List<Arc<T,LocatedNode<T>>> newArcs = new ArrayList<>();
        NodeList arcsNodeList = document.getElementsByTagName("arc");
        for (int i = 0; i < arcsNodeList.getLength(); i++) {
            Node arcNode = arcsNodeList.item(i);
            if(arcNode.getNodeType() == Node.ELEMENT_NODE) {
                Element arcElement = (Element) arcNode;
                Arc<T, LocatedNode<T>> newArc = new Arc<>();
                int edge1Id = Integer.parseInt(arcElement.getAttribute("edge1"));
                int edge2Id = Integer.parseInt(arcElement.getAttribute("edge2"));
                newArc.edge1 = newNodesMap.get(edge1Id);
                newArc.edge2 = newNodesMap.get(edge2Id);
                newArc.edge1.arcs.add(newArc);
                newArc.edge2.arcs.add(newArc);
                newArcs.add(newArc);
            }
        }

        nodes = newNodes;
        arcs = newArcs;
        selectedArc = null;
        selectedNode = null;
    }

    public interface TextContentParser<TC> {

        public TC parseTextContent(String textContent);
    }
}
