package swinggrapheditor.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.event.*;
import java.sql.SQLException;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.text.BadLocationException;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;
// http://fifesoft.com/rsyntaxtextarea/
import org.fife.ui.rsyntaxtextarea.RSyntaxDocument;
import org.fife.ui.rsyntaxtextarea.RSyntaxTextArea; 
import org.fife.ui.rsyntaxtextarea.SyntaxConstants;
import org.fife.ui.rtextarea.RTextScrollPane;
import swinggrapheditor.graph.GraphListener;
import swinggrapheditor.graph.xml.LocatedGraphXml;

/**
 *
 * @author trust56
 */
public class GraphEditorFrame extends JFrame {

    SwingGraph<Character> graph;

    public GraphEditorFrame(String nickname) {
        setTitle("Gráf szerkesztő");
        setLocation(40, 40);
        setSize(800, 400);
        addWindowListener(windowAdapter);
        graphPanel.setDoubleBuffered(true);
        graphPanel.addMouseListener(graphPanelMouseAdapter);
        graphPanel.addMouseMotionListener(graphPanelMouseAdapter);
        graphPanel.setFocusable(true);
        graphPanel.addKeyListener(graphPanelKeyListener);
        graph = new SwingGraph<>(nickname);
        graph.addGraphListener(new GraphListener() {
            @Override
            public void graphChanged() {
                graphPanel.repaint();
                updateXmlArea();
            }
        });
        graph.setTextContentParser(new LocatedGraphXml.TextContentParser<Character>() {

            @Override
            public Character parseTextContent(String textContent) {
                return textContent.charAt(0);
            }
        });
        xmlArea.setEditable(false);
        getContentPane().add(graphPanel, BorderLayout.CENTER);
        RTextScrollPane rTextScrollPane = new RTextScrollPane(xmlArea);
        rTextScrollPane.setPreferredSize(new Dimension(400, 100));
        getContentPane().add(rTextScrollPane, BorderLayout.EAST);
        graph.loadFromDataSource();
    }
    private WindowListener windowAdapter = new WindowAdapter() {
        @Override
        public void windowClosing(WindowEvent e) {
            GraphEditorFrame.this.dispose();
        }
    };
    private JPanel graphPanel = new JPanel() {
        @Override
        public void paint(Graphics g) {
            g.setColor(Color.LIGHT_GRAY);
            g.fillRect(0, 0, getWidth(), getHeight());
            if (mouseDraggedPoint != null) {
                g.setColor(Color.BLUE);
                int x1 = mousePressedPoint.x;
                int y1 = mousePressedPoint.y;
                int x2 = mouseDraggedPoint.x;
                int y2 = mouseDraggedPoint.y;
                g.drawLine(x1, y1, x2, y2);
            }
            graph.paint(g);
        }
    };
    private RSyntaxDocument xmlAreaDocument = new RSyntaxDocument(SyntaxConstants.SYNTAX_STYLE_XML);
    private RSyntaxTextArea xmlArea = new RSyntaxTextArea(xmlAreaDocument);
    private Point mousePressedPoint = new Point(0, 0);
    private Point mouseDraggedPoint = null;
    private MouseAdapter graphPanelMouseAdapter = new MouseAdapter() {
        private final int DISTANCE_TRESHOLD = 10;
        private boolean mouseDragged = false;
        private int mousePressedButton;

        @Override
        public void mouseClicked(MouseEvent e) {
            if (e.getButton() == MouseEvent.BUTTON1) {
                int x = e.getX();
                int y = e.getY();
                graph.addNode(x, y, ' ');
            } else if (e.getButton() == MouseEvent.BUTTON3) {
                graph.addArc(Utils.p2p(e.getPoint()));
            }
        }

        @Override
        public void mousePressed(MouseEvent e) {
            mouseDragged = false;
            mousePressedPoint = e.getPoint();
            mousePressedButton = e.getButton();
        }

        @Override
        public void mouseMoved(MouseEvent e) {
            graph.selectPoint(Utils.p2p(e.getPoint()));
        }

        @Override
        public void mouseDragged(MouseEvent e) {
            mouseDragged = !SwingGraph.pointsCloserThan(DISTANCE_TRESHOLD,
                    Utils.p2p(e.getPoint()), Utils.p2p(mousePressedPoint));
            if (mousePressedButton == MouseEvent.BUTTON1) {
                graph.dragPoint(Utils.p2p(e.getPoint()));
            } else if (mousePressedButton == MouseEvent.BUTTON3) {
                mouseDraggedPoint = e.getPoint();
                graphPanel.repaint();
            }
        }

        @Override
        public void mouseReleased(MouseEvent e) {
            if (mouseDragged) {
                if (e.getButton() == MouseEvent.BUTTON3) {
                    graph.addArc(Utils.p2p(mouseDraggedPoint));
                }
            } else {
                mouseClicked(e);
            }
            mouseDraggedPoint = null;
            graphPanel.repaint();
        }
    };
    private KeyListener graphPanelKeyListener = new KeyAdapter() {
        @Override
        public void keyReleased(KeyEvent e) {
            if (e.getKeyCode() == KeyEvent.VK_DELETE) {
                graph.deleteSelectedNodeOrArc();
            } else {
                graph.setElementInSelectedNode(e.getKeyChar());
            }
        }
    };

    private void updateXmlArea() {
        new Thread() {
            @Override
            public void run() {
                String xml;
                try {
                    xml = graph.saveToDataSource();
                } catch (ParserConfigurationException | TransformerException | SQLException ex) {
                    EventQueue.invokeLater(new Runnable() {
                        @Override
                        public void run() {
                            JOptionPane.showMessageDialog(rootPane, "Cannot convert the graph to xml or save it", "Error", JOptionPane.ERROR_MESSAGE);
                            ex.printStackTrace(System.err);
                        }
                    });
                    return;
                }
                final String xmll = xml;
                EventQueue.invokeLater(new Runnable() {
                    @Override
                    public void run() {
                        try {
                            xmlAreaDocument.replace(0, xmlAreaDocument.getLength(), xmll, null);
                        } catch (BadLocationException ex) {
                        }
                    }
                });
            }
        }.start();
    }
}
